searchData={"items":[{"type":"module","title":"EctoRange","doc":"`EctoRange` is a tiny library that provides Ecto custom types for all [Postgres Range types](https://www.postgresql.org/docs/current/rangetypes.html).\n\nThe main design goal of `EctoRange` is to be easy to use with native Elixir types,\nwhile providing the full flexibility of Postgres Range types inside of Ecto.\n\n| Postgres type | `Ecto.Type`             |\n| ------------- | ----------------------- |\n| int4range     | `EctoRange.Int4`        |\n| int8range     | `EctoRange.Int8`        |\n| numrange      | `EctoRange.Num`         |\n| tsrange       | `EctoRange.Timestamp`   |\n| tstzrange     | `EctoRange.TimestampTZ` |\n| daterange     | `EctoRange.Date`        |","ref":"EctoRange.html"},{"type":"module","title":"Usage - EctoRange","doc":"`EctoRange` provides custom types, and can be used like any other Ecto type.\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddRange do\n  use Ecto.Migration\n\n  def change do\n    create table(:my_table) do\n      add :name, :string\n      add :range, :daterange\n    end\n  end\nend\n\ndefmodule MyApp.Table do\n  use Ecto.Schema\n\n  import Ecto.Changeset\n\n  schema \"my_table\" do\n    field(:name, :string)\n    field(:range, EctoRange.Date)\n  end\n\n  def changeset(table, params) do\n    table\n    |> cast(params, [:name, :range])\n    |> validate_required([:name, :range])\n  end\nend\n\niex> alias MyApp.Table\n\niex> range = Date.range(~D[1989-09-22], ~D[2021-03-01])\n\niex> cs = Table.changeset(%Table{}, %{name: \"table\", range: range})\n\niex> cs.changes\n%Ecto.Changeset{\n  changes: %{\n    range: %Postgrex.Range{\n      lower: ~D[1989-09-22],\n      upper: ~D[2021-03-01],\n      lower_inclusive: true,\n      upper_inclusive: true\n    },\n    name: \"name\"\n  },\n  data: %MyApp.Table{id: nil, name: nil, range: nil},\n  params: %{\n    \"range\" => Date.range(~D[1989-09-22], ~D[2021-03-01]),\n    \"name\" => \"name\"\n  },\n  valid?: true\n}\n```","ref":"EctoRange.html#module-usage"},{"type":"module","title":"Installation - EctoRange","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `ectorange` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ecto_range, \"~> 0.1.0\"}\n  ]\nend\n```\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"EctoRange.html#module-installation"},{"type":"module","title":"Alternative libraries - EctoRange","doc":"If you're unhappy with the feature set`EctoRange`, there are some alternative libraries that provide similar functionality\n\n- [ecto_date_time_range](https://github.com/synchronal/ecto_date_time_range)\n- [pg_ranges](https://github.com/vforgione/pg_ranges)","ref":"EctoRange.html#module-alternative-libraries"},{"type":"module","title":"EctoRange.Date","doc":"A custom type for working with the Postgres [daterange](https://www.postgresql.org/docs/current/rangetypes.html#RANGETYPES-BUILTIN) type.\n\n    iex> range = Date.range(~D[1989-09-22], ~D[2021-03-01])\n    iex> cs = TestApp.Table.changeset(%TestApp.Table{name: \"EctoRange.Date\"}, %{range: range})\n    iex> cs.changes\n    %{range: %Postgrex.Range{lower: ~D[1989-09-22], upper: ~D[2021-03-01], lower_inclusive: true, upper_inclusive: true}}","ref":"EctoRange.Date.html"},{"type":"module","title":"Casting - EctoRange.Date","doc":"`EctoRange.Date` provides a couple of conveniences when casting data. All valid\ndata will be cast into a `t:Postgrex.Range.t/0` struct. When supplied to an Ecto.Changeset,\nthe following types are valid\n\n* `t:Date.Range.t/0` will be treated as an inclusive range\n* `{t:date() | t:String.t/0, t:date() | t:String.t/0}` can be used to express unbounded ranges,\nwhere `nil` represents an unbounded endpoint\n* `t:Postgrex.Range.t/0` will be treated as a valid range representation","ref":"EctoRange.Date.html#module-casting"},{"type":"module","title":"Loading - EctoRange.Date","doc":"All data loaded from the database will be normalized into an inclusive range\nto align with the semantics of `Date.Range.t()`","ref":"EctoRange.Date.html#module-loading"},{"type":"type","title":"EctoRange.Date.date/0","doc":"","ref":"EctoRange.Date.html#t:date/0"},{"type":"function","title":"EctoRange.Date.embed_as/1","doc":"","ref":"EctoRange.Date.html#embed_as/1"},{"type":"function","title":"EctoRange.Date.equal?/2","doc":"","ref":"EctoRange.Date.html#equal?/2"},{"type":"function","title":"EctoRange.Date.normalize_range/1","doc":"Converts a Postgrex.Range.t() into a normalized form. For bounded ranges,\nit will make the lower and upper bounds inclusive to align with the semantics\nof Date.Range.t()\n\n    iex> range = %Postgrex.Range{lower: ~D[1989-09-22], upper: ~D[2021-03-02], lower_inclusive: true, upper_inclusive: false}\n    iex> EctoRange.Date.normalize_range(range)\n    %Postgrex.Range{lower: ~D[1989-09-22], upper: ~D[2021-03-01], lower_inclusive: true, upper_inclusive: true}\n\n    iex> range = %Postgrex.Range{lower: ~D[1989-09-21], upper: ~D[2021-03-01], lower_inclusive: false, upper_inclusive: true}\n    iex> EctoRange.Date.normalize_range(range)\n    %Postgrex.Range{lower: ~D[1989-09-22], upper: ~D[2021-03-01], lower_inclusive: true, upper_inclusive: true}","ref":"EctoRange.Date.html#normalize_range/1"},{"type":"function","title":"EctoRange.Date.to_postgrex_range/1","doc":"Converts valid `Date.Range.t()` or `Date.t()` tuples into a `Postgrex.Range.t()`\n\n    iex> EctoRange.Date.to_postgrex_range(Date.range(~D[1989-09-22], ~D[2021-03-01]))\n    %Postgrex.Range{lower: ~D[1989-09-22], upper: ~D[2021-03-01], lower_inclusive: true, upper_inclusive: true}","ref":"EctoRange.Date.html#to_postgrex_range/1"},{"type":"module","title":"EctoRange.Int4","doc":"A Postgres range of `int4` integers. Equivalent to `int4range`.","ref":"EctoRange.Int4.html"},{"type":"function","title":"EctoRange.Int4.embed_as/1","doc":"","ref":"EctoRange.Int4.html#embed_as/1"},{"type":"function","title":"EctoRange.Int4.equal?/2","doc":"","ref":"EctoRange.Int4.html#equal?/2"},{"type":"function","title":"EctoRange.Int4.normalize_range/1","doc":"Converts a Postgrex.Range.t() into a normalized form. For bounded ranges,\nit will make the lower and upper bounds inclusive.","ref":"EctoRange.Int4.html#normalize_range/1"},{"type":"function","title":"Examples - EctoRange.Int4.normalize_range/1","doc":"iex> range = %Postgrex.Range{lower: 1, upper: 3, lower_inclusive: true, upper_inclusive: false}\n    iex> EctoRange.Int4.normalize_range(range)\n    %Postgrex.Range{lower: 1, upper: 2, lower_inclusive: true, upper_inclusive: true}\n\n    iex> range = %Postgrex.Range{lower: 1, upper: 3, lower_inclusive: false, upper_inclusive: true}\n    iex> EctoRange.Int4.normalize_range(range)\n    %Postgrex.Range{lower: 2, upper: 3, lower_inclusive: true, upper_inclusive: true}","ref":"EctoRange.Int4.html#normalize_range/1-examples"},{"type":"function","title":"EctoRange.Int4.to_postgrex_range/1","doc":"Checks and converts a `Postgrex.Range` or tuple into a `Postgrex.Range.t()`","ref":"EctoRange.Int4.html#to_postgrex_range/1"},{"type":"function","title":"Examples - EctoRange.Int4.to_postgrex_range/1","doc":"iex> EctoRange.Int4.to_postgrex_range({1, 3})\n    %Postgrex.Range{lower: 1, upper: 3, lower_inclusive: true, upper_inclusive: true}","ref":"EctoRange.Int4.html#to_postgrex_range/1-examples"},{"type":"module","title":"EctoRange.Int8","doc":"A Postgres range of `int8` integers. Equivalent to `int8range`.\nAllows `bigint` values.","ref":"EctoRange.Int8.html"},{"type":"function","title":"EctoRange.Int8.embed_as/1","doc":"","ref":"EctoRange.Int8.html#embed_as/1"},{"type":"function","title":"EctoRange.Int8.equal?/2","doc":"","ref":"EctoRange.Int8.html#equal?/2"},{"type":"function","title":"EctoRange.Int8.normalize_range/1","doc":"Converts a Postgrex.Range.t() into a normalized form. For bounded ranges,\nit will make the lower and upper bounds inclusive.","ref":"EctoRange.Int8.html#normalize_range/1"},{"type":"function","title":"Examples - EctoRange.Int8.normalize_range/1","doc":"iex> range = %Postgrex.Range{lower: 1, upper: 3, lower_inclusive: true, upper_inclusive: false}\n    iex> EctoRange.Int8.normalize_range(range)\n    %Postgrex.Range{lower: 1, upper: 2, lower_inclusive: true, upper_inclusive: true}\n\n    iex> range = %Postgrex.Range{lower: 1, upper: 3, lower_inclusive: false, upper_inclusive: true}\n    iex> EctoRange.Int8.normalize_range(range)\n    %Postgrex.Range{lower: 2, upper: 3, lower_inclusive: true, upper_inclusive: true}","ref":"EctoRange.Int8.html#normalize_range/1-examples"},{"type":"function","title":"EctoRange.Int8.to_postgrex_range/1","doc":"Checks and converts a `Postgrex.Range` or tuple into a `Postgrex.Range.t()`","ref":"EctoRange.Int8.html#to_postgrex_range/1"},{"type":"function","title":"Examples - EctoRange.Int8.to_postgrex_range/1","doc":"iex> EctoRange.Int8.to_postgrex_range({1, 3})\n    %Postgrex.Range{lower: 1, upper: 3, lower_inclusive: true, upper_inclusive: true}","ref":"EctoRange.Int8.html#to_postgrex_range/1-examples"},{"type":"module","title":"EctoRange.Num","doc":"A Postgres range of `numeric` values. Equivalent to `numrange`.\nAllows numeric values of all precisions.\nReturns `Decimal` structs when loading from the database.\n\nWatch out: Not defining the precision of your values,\nmight cause compatibility problems with other systems in the future.","ref":"EctoRange.Num.html"},{"type":"function","title":"EctoRange.Num.embed_as/1","doc":"","ref":"EctoRange.Num.html#embed_as/1"},{"type":"function","title":"EctoRange.Num.equal?/2","doc":"","ref":"EctoRange.Num.html#equal?/2"},{"type":"function","title":"EctoRange.Num.normalize_range/1","doc":"Converts a Postgrex.Range.t() into a normalized form. For bounded ranges,\nit will make the lower and upper bounds inclusive.\n\nAll upper and lower bounds are converted to `%Decimal{}` structs, if necessary.","ref":"EctoRange.Num.html#normalize_range/1"},{"type":"function","title":"Examples - EctoRange.Num.normalize_range/1","doc":"iex> range = %Postgrex.Range{lower: 1, upper: 3, lower_inclusive: true, upper_inclusive: false}\n    iex> EctoRange.Num.normalize_range(range)\n    %Postgrex.Range{lower: %Decimal{coef: 1}, upper: %Decimal{coef: 2999999999, exp: -9}, lower_inclusive: true, upper_inclusive: true}\n\n    iex> range = %Postgrex.Range{lower: 1, upper: 3, lower_inclusive: false, upper_inclusive: true}\n    iex> EctoRange.Num.normalize_range(range)\n    %Postgrex.Range{lower: %Decimal{coef: 1000000001, exp: -9}, upper: %Decimal{coef: 3}, lower_inclusive: true, upper_inclusive: true}","ref":"EctoRange.Num.html#normalize_range/1-examples"},{"type":"function","title":"EctoRange.Num.to_postgrex_range/1","doc":"Checks and converts a `Postgrex.Range` or tuple into a `Postgrex.Range.t()`","ref":"EctoRange.Num.html#to_postgrex_range/1"},{"type":"function","title":"Examples - EctoRange.Num.to_postgrex_range/1","doc":"iex> EctoRange.Date.to_postgrex_range({1, 3})\n    %Postgrex.Range{lower: 1, upper: 3, lower_inclusive: true, upper_inclusive: true}","ref":"EctoRange.Num.html#to_postgrex_range/1-examples"},{"type":"module","title":"EctoRange.Timestamp","doc":"A custom type for working with the Postgres [tsrange](https://www.postgresql.org/docs/current/rangetypes.html#RANGETYPES-BUILTIN) type.\n\n    iex> range = {~N[2020-10-31 09:30:00], ~N[2020-11-02 10:00:00]}\n    iex> cs = TestApp.Table.changeset(%TestApp.Table{name: \"EctoRange.Timestamp\"}, %{tsrange: range})\n    iex> cs.changes\n    %{tsrange: %Postgrex.Range{lower: ~N[2020-10-31 09:30:00.000000], upper: ~N[2020-11-02 10:00:00.000000], lower_inclusive: true, upper_inclusive: true}}\n\n`tsrange` in Postgres is a continuous range, and does not have any equivalent Elixir struct.","ref":"EctoRange.Timestamp.html"},{"type":"module","title":"Casting - EctoRange.Timestamp","doc":"`EctoRange.Timestamp` provides a couple of conveniences when casting data. All valid\ndata will be cast into a `t:Postgrex.Range.t/0` struct. When supplied to an Ecto.Changeset,\nthe following types are valid\n\n* `{t:timestamp() | t:String.t/0, t:timestamp() | t:String.t/0}` can be used to express unbounded ranges,\nwhere `nil` represents an unbounded endpoint\n* `t:Postgrex.Range.t/0` will be treated as a valid range representation","ref":"EctoRange.Timestamp.html#module-casting"},{"type":"module","title":"Loading - EctoRange.Timestamp","doc":"All data loaded from the database will be normalized into an inclusive range\nto align with the semantics of `Date.Range.t()`","ref":"EctoRange.Timestamp.html#module-loading"},{"type":"function","title":"EctoRange.Timestamp.embed_as/1","doc":"","ref":"EctoRange.Timestamp.html#embed_as/1"},{"type":"function","title":"EctoRange.Timestamp.equal?/2","doc":"","ref":"EctoRange.Timestamp.html#equal?/2"},{"type":"type","title":"EctoRange.Timestamp.timestamp/0","doc":"","ref":"EctoRange.Timestamp.html#t:timestamp/0"},{"type":"function","title":"EctoRange.Timestamp.to_postgrex_range/1","doc":"Converts valid `NaiveDateTime.t()` tuples into a `Postgrex.Range.t()`\n\n    iex> EctoRange.Date.to_postgrex_range({~N[2021-03-01 08:30:00], ~N[2023-03-30 10:30:00]})\n    %Postgrex.Range{lower: ~N[2021-03-01 08:30:00], upper: ~N[2023-03-30 10:30:00], lower_inclusive: true, upper_inclusive: true}","ref":"EctoRange.Timestamp.html#to_postgrex_range/1"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]\n\n## [0.3.0]","ref":"changelog.html"},{"type":"extras","title":"Changed - Changelog","doc":"- `EctoRange.Timestamp` no longer tries to normalize timestamps, and casts with `:naive_datetime_usec`\n- Ugraded dependencies\n\n## [0.2.1]","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Loosen up `postgrex` dependency requirements\n\n## [0.2.0]","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Support for `tsrange` via `EctoRange.Timestamp`\n- Example of `EctoRange.Date` in the README\n\n## [0.1.0] - 2023-03-25\n\nInitial release\n\n[unreleased]: https://github.com/davydog187/ecto_range/compare/v0.3.0...HEAD\n[0.3.0]: https://github.com/davydog187/ecto_range/compare/v0.2.1..v0.3.0\n[0.2.1]: https://github.com/davydog187/ecto_range/compare/v0.2.0..v0.2.1\n[0.2.0]: https://github.com/davydog187/ecto_range/compare/v0.1.0..v0.2.0\n[0.1.0]: https://github.com/davydog187/ecto_range/releases/tag/v0.1.0","ref":"changelog.html#added"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}